pub struct BaseGraph<K, D> {
pub sequences: PackedDnaStringSet,
pub exts: Vec<Exts>,
pub data: Vec<D>,
pub stranded: bool,
/* private fields */
}Expand description
A compressed DeBruijn graph carrying auxiliary data on each node of type D.
This type does not carry the sorted index arrays the allow the graph
to be walked efficiently. The DeBruijnGraph type wraps this type and add those
vectors.
Fields
sequences: PackedDnaStringSetexts: Vec<Exts>data: Vec<D>stranded: boolImplementations
Sourceimpl<K: Kmer + Send + Sync, D> BaseGraph<K, D>
impl<K: Kmer + Send + Sync, D> BaseGraph<K, D>
pub fn finish(self) -> DebruijnGraph<K, D>
Sourceimpl<K: Kmer, D> BaseGraph<K, D>
impl<K: Kmer, D> BaseGraph<K, D>
pub fn finish_serial(self) -> DebruijnGraph<K, D>