Node

Struct Node 

pub struct Node<'a, K: Kmer + 'a, D: 'a> {
    pub node_id: usize,
    pub graph: &'a DebruijnGraph<K, D>,
}
Unbranched sequence in the DeBruijn graph

Fields

node_id: usizegraph: &'a DebruijnGraph<K, D>

Implementations

Source

impl<'a, K: Kmer, D: Debug> Node<'a, K, D>

pub fn len(&self) -> usize

Length of the sequence of this node

pub fn is_empty(&self) -> bool

pub fn sequence(&self) -> DnaStringSlice<'a>

Sequence of the node

pub fn data(&self) -> &'a D

Reference to auxiliarly data associated with the node

pub fn exts(&self) -> Exts

Extension bases from this node

pub fn l_edges(&self) -> SmallVec<[(u8, usize, Dir, bool); 4]>

Edges leaving the left side of the node in the format (base, target_node id, incoming side of target node, whether target node is flipped)

pub fn r_edges(&self) -> SmallVec<[(u8, usize, Dir, bool); 4]>

Edges leaving the right side of the node in the format (base, target_node id, incoming side of target node, whether target node is flipped)

pub fn edges(&self, dir: Dir) -> SmallVec<[(u8, usize, Dir, bool); 4]>

Edges leaving the ‘dir’ side of the node in the format (base, target_node id, incoming side of target node, whether target node is flipped)

Source

impl<K: Kmer, SD: Debug> Node<'_, K, SD>

pub fn edge_dot_default<DI>( &self, colors: &Colors<'_, SD, DI>, base: u8, incoming_dir: Dir, flipped: bool, ) -> String
where SD: SummaryData<DI>,

get default format for dot edges based on node data

pub fn node_dot_default<DI>( &self, colors: &Colors<'_, SD, DI>, config: &SummaryConfig, translator: &Translator, outline: bool, translate_id_groups: bool, ) -> String
where SD: SummaryData<DI>,

get default format for dot nodes, based on node data

pub fn edge_json_default<DI>( &self, target_node_id: usize, base: u8, incoming_dir: Dir, flipped: bool, ) -> String
where SD: SummaryData<DI>,

pub fn node_json_default<DI>( &self, colors: &Colors<'_, SD, DI>, config: &SummaryConfig, translator: &Translator, translate_id_groups: bool, ) -> String
where SD: SummaryData<DI>,

get default properties for json nodes, based on node data