pub struct Node<'a, K: Kmer + 'a, D: 'a> {
pub node_id: usize,
pub graph: &'a DebruijnGraph<K, D>,
}Expand description
Unbranched sequence in the DeBruijn graph
Fields
node_id: usizegraph: &'a DebruijnGraph<K, D>Implementations
Sourceimpl<'a, K: Kmer, D: Debug> Node<'a, K, D>
impl<'a, K: Kmer, D: Debug> Node<'a, K, D>
pub fn is_empty(&self) -> bool
Sourcepub fn sequence(&self) -> DnaStringSlice<'a>
pub fn sequence(&self) -> DnaStringSlice<'a>
Sequence of the node
Sourcepub fn l_edges(&self) -> SmallVec<[(u8, usize, Dir, bool); 4]>
pub fn l_edges(&self) -> SmallVec<[(u8, usize, Dir, bool); 4]>
Edges leaving the left side of the node in the format (base, target_node id, incoming side of target node, whether target node is flipped)
Sourceimpl<K: Kmer, SD: Debug> Node<'_, K, SD>
impl<K: Kmer, SD: Debug> Node<'_, K, SD>
Sourcepub fn edge_dot_default<DI>(
&self,
colors: &Colors<'_, SD, DI>,
base: u8,
incoming_dir: Dir,
flipped: bool,
) -> Stringwhere
SD: SummaryData<DI>,
pub fn edge_dot_default<DI>(
&self,
colors: &Colors<'_, SD, DI>,
base: u8,
incoming_dir: Dir,
flipped: bool,
) -> Stringwhere
SD: SummaryData<DI>,
get default format for dot edges based on node data
Sourcepub fn node_dot_default<DI>(
&self,
colors: &Colors<'_, SD, DI>,
config: &SummaryConfig,
translator: &Translator,
outline: bool,
translate_id_groups: bool,
) -> Stringwhere
SD: SummaryData<DI>,
pub fn node_dot_default<DI>(
&self,
colors: &Colors<'_, SD, DI>,
config: &SummaryConfig,
translator: &Translator,
outline: bool,
translate_id_groups: bool,
) -> Stringwhere
SD: SummaryData<DI>,
get default format for dot nodes, based on node data
pub fn edge_json_default<DI>(
&self,
target_node_id: usize,
base: u8,
incoming_dir: Dir,
flipped: bool,
) -> Stringwhere
SD: SummaryData<DI>,
Sourcepub fn node_json_default<DI>(
&self,
colors: &Colors<'_, SD, DI>,
config: &SummaryConfig,
translator: &Translator,
translate_id_groups: bool,
) -> Stringwhere
SD: SummaryData<DI>,
pub fn node_json_default<DI>(
&self,
colors: &Colors<'_, SD, DI>,
config: &SummaryConfig,
translator: &Translator,
translate_id_groups: bool,
) -> Stringwhere
SD: SummaryData<DI>,
get default properties for json nodes, based on node data